/*
 *  vtgconfigurationdialog.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgconfiguration.h"
#include <gconf/gconf-client.h>
#include <stdlib.h>
#include <string.h>
#include <gconf/gconf.h>
#include <glib/gi18n-lib.h>
#include "vtgutils.h"




struct _VtgConfigurationPrivate {
	GConfClient* _gconf;
	GtkDialog* _dialog;
	gboolean _bracket_enabled;
	gboolean _symbol_enabled;
};

#define VTG_CONFIGURATION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_CONFIGURATION, VtgConfigurationPrivate))
enum  {
	VTG_CONFIGURATION_DUMMY_PROPERTY,
	VTG_CONFIGURATION_BRACKET_ENABLED,
	VTG_CONFIGURATION_SYMBOL_ENABLED,
	VTG_CONFIGURATION_SAVE_BEFORE_BUILD
};
#define VTG_CONFIGURATION_VTG_BASE_KEY "/apps/gedit-2/plugins/vtg"
#define VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY VTG_CONFIGURATION_VTG_BASE_KEY "/bracket_completion_enabled"
#define VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY VTG_CONFIGURATION_VTG_BASE_KEY "/symbol_completion_enabled"
static void _vtg_configuration_on_button_close_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self);
static void _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled (GtkCheckButton* _sender, gpointer self);
static void vtg_configuration_on_button_close_clicked (VtgConfiguration* self, GtkButton* sender);
static void vtg_configuration_on_conf_value_changed (VtgConfiguration* self, GConfClient* sender, const char* key, void* value);
static void vtg_configuration_on_checkbutton_toggled (VtgConfiguration* self, GtkToggleButton* sender);
static void _vtg_configuration_on_conf_value_changed_gconf_client_value_changed (GConfClient* _sender, const char* key, void* value, gpointer self);
static GObject * vtg_configuration_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_configuration_parent_class = NULL;
static void vtg_configuration_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



static void _vtg_configuration_on_button_close_clicked_gtk_button_clicked (GtkButton* _sender, gpointer self) {
	vtg_configuration_on_button_close_clicked (self, _sender);
}


static void _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled (GtkCheckButton* _sender, gpointer self) {
	vtg_configuration_on_checkbutton_toggled (self, _sender);
}


GtkWidget* vtg_configuration_get_configuration_dialog (VtgConfiguration* self) {
	GError * inner_error;
	g_return_val_if_fail (self != NULL, NULL);
	inner_error = NULL;
	{
		GtkBuilder* builder;
		char* _tmp0;
		GtkDialog* _tmp2;
		GtkDialog* _tmp1;
		GtkButton* _tmp3;
		GtkButton* button;
		GtkCheckButton* _tmp4;
		GtkCheckButton* check;
		GtkCheckButton* _tmp6;
		GtkCheckButton* _tmp5;
		GtkWidget* _tmp7;
		builder = gtk_builder_new ();
		_tmp0 = NULL;
		gtk_builder_add_from_file (builder, _tmp0 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
		if (inner_error != NULL) {
			goto __catch22_g_error;
		}
		_tmp0 = (g_free (_tmp0), NULL);
		_tmp2 = NULL;
		_tmp1 = NULL;
		self->priv->_dialog = (_tmp2 = (_tmp1 = GTK_DIALOG (gtk_builder_get_object (builder, "dialog-settings")), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL)), _tmp2);
		g_assert (self->priv->_dialog != NULL);
		_tmp3 = NULL;
		button = (_tmp3 = GTK_BUTTON (gtk_builder_get_object (builder, "button-settings-close")), (_tmp3 == NULL) ? NULL : g_object_ref (_tmp3));
		g_signal_connect_object (button, "clicked", (GCallback) _vtg_configuration_on_button_close_clicked_gtk_button_clicked, self, 0);
		_tmp4 = NULL;
		check = (_tmp4 = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "checkbutton-settings-bracket-completion")), (_tmp4 == NULL) ? NULL : g_object_ref (_tmp4));
		g_assert (check != NULL);
		gtk_toggle_button_set_active ((GtkToggleButton*) check, self->priv->_bracket_enabled);
		g_signal_connect_object ((GtkToggleButton*) check, "toggled", (GCallback) _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled, self, 0);
		_tmp6 = NULL;
		_tmp5 = NULL;
		check = (_tmp6 = (_tmp5 = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "checkbutton-settings-symbol-completion")), (_tmp5 == NULL) ? NULL : g_object_ref (_tmp5)), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp6);
		g_assert (check != NULL);
		gtk_toggle_button_set_active ((GtkToggleButton*) check, self->priv->_symbol_enabled);
		g_signal_connect_object ((GtkToggleButton*) check, "toggled", (GCallback) _vtg_configuration_on_checkbutton_toggled_gtk_toggle_button_toggled, self, 0);
		_tmp7 = NULL;
		return (_tmp7 = (GtkWidget*) self->priv->_dialog, (builder == NULL) ? NULL : (builder = (g_object_unref (builder), NULL)), (button == NULL) ? NULL : (button = (g_object_unref (button), NULL)), (check == NULL) ? NULL : (check = (g_object_unref (check), NULL)), _tmp7);
	}
	goto __finally22;
	__catch22_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			GtkWidget* _tmp8;
			g_warning ("vtgconfiguration.vala:116: (get_configuration_dialog): %s", err->message);
			_tmp8 = NULL;
			return (_tmp8 = NULL, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), _tmp8);
		}
	}
	__finally22:
	;
}


static void vtg_configuration_on_button_close_clicked (VtgConfiguration* self, GtkButton* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (self->priv->_dialog != NULL);
	gtk_object_destroy ((GtkObject*) self->priv->_dialog);
}


static void vtg_configuration_on_conf_value_changed (VtgConfiguration* self, GConfClient* sender, const char* key, void* value) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (key != NULL);
	inner_error = NULL;
	{
		if (_vala_strcmp0 (key, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY) == 0) {
			gboolean new_val;
			new_val = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch23_g_error;
			}
			if (self->priv->_bracket_enabled != new_val) {
				vtg_configuration_set_bracket_enabled (self, new_val);
			}
		} else {
			if (_vala_strcmp0 (key, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY) == 0) {
				gboolean new_val;
				new_val = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
				if (inner_error != NULL) {
					goto __catch23_g_error;
				}
				if (self->priv->_symbol_enabled != new_val) {
					vtg_configuration_set_symbol_enabled (self, new_val);
				}
			}
		}
	}
	goto __finally23;
	__catch23_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgconfiguration.vala:142: (on_conf_value_changed): %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally23:
	;
}


static void vtg_configuration_on_checkbutton_toggled (VtgConfiguration* self, GtkToggleButton* sender) {
	GError * inner_error;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	inner_error = NULL;
	{
		gboolean new_val;
		const char* _tmp0;
		char* name;
		new_val = gtk_toggle_button_get_active (sender);
		_tmp0 = NULL;
		name = (_tmp0 = gtk_buildable_get_name ((GtkBuildable*) sender), (_tmp0 == NULL) ? NULL : g_strdup (_tmp0));
		if (_vala_strcmp0 (name, "checkbutton-settings-bracket-completion") == 0) {
			gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, new_val, &inner_error);
			if (inner_error != NULL) {
				goto __catch24_g_error;
			}
		} else {
			if (_vala_strcmp0 (name, "checkbutton-settings-symbol-completion") == 0) {
				gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, new_val, &inner_error);
				if (inner_error != NULL) {
					goto __catch24_g_error;
				}
			}
		}
		name = (g_free (name), NULL);
	}
	goto __finally24;
	__catch24_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning ("vtgconfiguration.vala:158: (on_checkbutton_toggled): %s", err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally24:
	;
}


VtgConfiguration* vtg_configuration_construct (GType object_type) {
	VtgConfiguration * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgConfiguration* vtg_configuration_new (void) {
	return vtg_configuration_construct (VTG_TYPE_CONFIGURATION);
}


gboolean vtg_configuration_get_bracket_enabled (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_bracket_enabled;
}


void vtg_configuration_set_bracket_enabled (VtgConfiguration* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_bracket_enabled = value;
	g_object_notify ((GObject *) self, "bracket-enabled");
}


gboolean vtg_configuration_get_symbol_enabled (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_symbol_enabled;
}


void vtg_configuration_set_symbol_enabled (VtgConfiguration* self, gboolean value) {
	g_return_if_fail (self != NULL);
	self->priv->_symbol_enabled = value;
	g_object_notify ((GObject *) self, "symbol-enabled");
}


gboolean vtg_configuration_get_save_before_build (VtgConfiguration* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return TRUE;
}


static void _vtg_configuration_on_conf_value_changed_gconf_client_value_changed (GConfClient* _sender, const char* key, void* value, gpointer self) {
	vtg_configuration_on_conf_value_changed (self, _sender, key, value);
}


/*TODO: implement me!*/
static GObject * vtg_configuration_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgConfigurationClass * klass;
	GObjectClass * parent_class;
	VtgConfiguration * self;
	GError * inner_error;
	klass = VTG_CONFIGURATION_CLASS (g_type_class_peek (VTG_TYPE_CONFIGURATION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_CONFIGURATION (obj);
	inner_error = NULL;
	{
		{
			GConfClient* _tmp1;
			GConfClient* _tmp0;
			gboolean _tmp2;
			gboolean _tmp3;
			gboolean _tmp4;
			gboolean _tmp5;
			g_debug ("vtgconfiguration.vala:51: starting");
			/*TODO: construct the gconf client from Engine.get_defualt ()
			when supported. See this bug for a similar issue
			http://bugzilla.gnome.org/show_bug.cgi?id=549061*/
			_tmp1 = NULL;
			_tmp0 = NULL;
			self->priv->_gconf = (_tmp1 = (_tmp0 = gconf_client_get_default (), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0)), (self->priv->_gconf == NULL) ? NULL : (self->priv->_gconf = (g_object_unref (self->priv->_gconf), NULL)), _tmp1);
			_tmp2 = gconf_client_dir_exists (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_BASE_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			if (!_tmp2) {
				GConfSchema* schema;
				GConfValue* def_value;
				g_debug ("vtgconfiguration.vala:57: creating configuration schemas");
				schema = gconf_schema_new ();
				gconf_schema_set_short_desc (schema, _ ("Enable the symbol completion module"));
				gconf_schema_set_type (schema, GCONF_VALUE_BOOL);
				def_value = gconf_value_new (GCONF_VALUE_BOOL);
				gconf_value_set_bool (def_value, TRUE);
				gconf_schema_set_default_value (schema, def_value);
				gconf_client_set_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, schema, &inner_error);
				if (inner_error != NULL) {
					goto __catch25_g_error;
				}
				gconf_schema_set_short_desc (schema, _ ("Enable the bracket completion module"));
				gconf_schema_set_type (schema, GCONF_VALUE_BOOL);
				gconf_schema_set_default_value (schema, def_value);
				gconf_client_set_schema (self->priv->_gconf, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, schema, &inner_error);
				if (inner_error != NULL) {
					goto __catch25_g_error;
				}
				(schema == NULL) ? NULL : (schema = (gconf_schema_free (schema), NULL));
				(def_value == NULL) ? NULL : (def_value = (gconf_value_free (def_value), NULL));
			}
			_tmp3 = gconf_client_dir_exists (self->priv->_gconf, VTG_CONFIGURATION_VTG_BASE_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			if (!_tmp3) {
				g_debug ("vtgconfiguration.vala:71: creating configuration keys");
				gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, TRUE, &inner_error);
				if (inner_error != NULL) {
					goto __catch25_g_error;
				}
				gconf_client_set_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, TRUE, &inner_error);
				if (inner_error != NULL) {
					goto __catch25_g_error;
				}
			}
			gconf_engine_associate_schema (self->priv->_gconf->engine, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			gconf_engine_associate_schema (self->priv->_gconf->engine, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, "/schemas" VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			_tmp4 = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_SYMBOL_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			self->priv->_symbol_enabled = _tmp4;
			_tmp5 = gconf_client_get_bool (self->priv->_gconf, VTG_CONFIGURATION_VTG_ENABLE_BRACKET_COMPLETION_KEY, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			self->priv->_bracket_enabled = _tmp5;
			;
			gconf_client_add_dir (self->priv->_gconf, VTG_CONFIGURATION_VTG_BASE_KEY, GCONF_CLIENT_PRELOAD_ONELEVEL, &inner_error);
			if (inner_error != NULL) {
				goto __catch25_g_error;
			}
			g_signal_connect_object (self->priv->_gconf, "value-changed", (GCallback) _vtg_configuration_on_conf_value_changed_gconf_client_value_changed, self, 0);
		}
		goto __finally25;
		__catch25_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgconfiguration.vala:82: (configuration): %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally25:
		;
	}
	return obj;
}


static void vtg_configuration_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgConfiguration * self;
	self = VTG_CONFIGURATION (object);
	switch (property_id) {
		case VTG_CONFIGURATION_BRACKET_ENABLED:
		g_value_set_boolean (value, vtg_configuration_get_bracket_enabled (self));
		break;
		case VTG_CONFIGURATION_SYMBOL_ENABLED:
		g_value_set_boolean (value, vtg_configuration_get_symbol_enabled (self));
		break;
		case VTG_CONFIGURATION_SAVE_BEFORE_BUILD:
		g_value_set_boolean (value, vtg_configuration_get_save_before_build (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_configuration_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgConfiguration * self;
	self = VTG_CONFIGURATION (object);
	switch (property_id) {
		case VTG_CONFIGURATION_BRACKET_ENABLED:
		vtg_configuration_set_bracket_enabled (self, g_value_get_boolean (value));
		break;
		case VTG_CONFIGURATION_SYMBOL_ENABLED:
		vtg_configuration_set_symbol_enabled (self, g_value_get_boolean (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_configuration_class_init (VtgConfigurationClass * klass) {
	vtg_configuration_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgConfigurationPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_configuration_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_configuration_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_configuration_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_configuration_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_BRACKET_ENABLED, g_param_spec_boolean ("bracket-enabled", "bracket-enabled", "bracket-enabled", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_SYMBOL_ENABLED, g_param_spec_boolean ("symbol-enabled", "symbol-enabled", "symbol-enabled", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_CONFIGURATION_SAVE_BEFORE_BUILD, g_param_spec_boolean ("save-before-build", "save-before-build", "save-before-build", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_configuration_instance_init (VtgConfiguration * self) {
	self->priv = VTG_CONFIGURATION_GET_PRIVATE (self);
}


static void vtg_configuration_finalize (GObject* obj) {
	VtgConfiguration * self;
	GError * inner_error;
	self = VTG_CONFIGURATION (obj);
	inner_error = NULL;
	{
		{
			gconf_client_suggest_sync (self->priv->_gconf, &inner_error);
			if (inner_error != NULL) {
				goto __catch26_g_error;
			}
		}
		goto __finally26;
		__catch26_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgconfiguration.vala:92: error %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally26:
		;
	}
	(self->priv->_gconf == NULL) ? NULL : (self->priv->_gconf = (g_object_unref (self->priv->_gconf), NULL));
	(self->priv->_dialog == NULL) ? NULL : (self->priv->_dialog = (g_object_unref (self->priv->_dialog), NULL));
	G_OBJECT_CLASS (vtg_configuration_parent_class)->finalize (obj);
}


static GType vtg_configuration_type_id = 0;
GType vtg_configuration_get_type (void) {
	return vtg_configuration_type_id;
}


GType vtg_configuration_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgConfigurationClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_configuration_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgConfiguration), 0, (GInstanceInitFunc) vtg_configuration_instance_init, NULL };
	vtg_configuration_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgConfiguration", &g_define_type_info, 0);
	return vtg_configuration_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




