/*
 *  vtgprojectmanagerbuilder.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgprojectmanagerbuilder.h"
#include <glib/gi18n-lib.h>
#include <sys/wait.h>
#include <gedit/gedit-window.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include "vtgprojectmanageroutputview.h"




static glong string_get_length (const char* self);
struct _VtgProjectManagerBuilderPrivate {
	VtgPlugin* _plugin;
	VtgProjectManagerBuildLogView* _build_view;
	guint _child_watch_id;
	gboolean is_bottom_pane_visible;
	gint last_exit_code;
};

#define VTG_PROJECT_MANAGER_BUILDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_PROJECT_MANAGER_TYPE_BUILDER, VtgProjectManagerBuilderPrivate))
enum  {
	VTG_PROJECT_MANAGER_BUILDER_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_BUILDER_PLUGIN,
	VTG_PROJECT_MANAGER_BUILDER_ERROR_PANE,
	VTG_PROJECT_MANAGER_BUILDER_IS_BUILDING
};
#define VTG_PROJECT_MANAGER_BUILDER_MAKE "make"
static GPid* _g_pid_dup (GPid* self);
static void _vtg_project_manager_builder_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self);
static void vtg_project_manager_builder_on_child_watch (VtgProjectManagerBuilder* self, GPid pid, gint status);
static void vtg_project_manager_builder_set_plugin (VtgProjectManagerBuilder* self, VtgPlugin* value);
static GObject * vtg_project_manager_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer vtg_project_manager_builder_parent_class = NULL;
static void vtg_project_manager_builder_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);



static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


VtgProjectManagerBuilder* vtg_project_manager_builder_construct (GType object_type, VtgPlugin* plugin) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerBuilder * self;
	g_return_val_if_fail (plugin != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN);
	g_value_set_object (&__params_it->value, plugin);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerBuilder* vtg_project_manager_builder_new (VtgPlugin* plugin) {
	return vtg_project_manager_builder_construct (VTG_PROJECT_MANAGER_TYPE_BUILDER, plugin);
}


static GPid* _g_pid_dup (GPid* self) {
	return g_memdup (self, sizeof (GPid));
}


static void _vtg_project_manager_builder_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self) {
	vtg_project_manager_builder_on_child_watch (self, pid, status);
}


gboolean vtg_project_manager_builder_compile_file (VtgProjectManagerBuilder* self, const char* filename) {
	GError * inner_error;
	char* working_dir;
	GPid* child_pid;
	gint stdo;
	gint stde;
	VtgProjectManagerOutputView* _tmp1;
	VtgProjectManagerOutputView* log;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (filename != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	working_dir = g_path_get_dirname (filename);
	child_pid = 0;
	stdo = 0;
	stde = 0;
	_tmp1 = NULL;
	log = (_tmp1 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
	{
		char* command;
		char* start_message;
		char* _tmp3;
		char* _tmp2;
		char* _tmp4;
		GPid* _tmp12;
		GPid* _tmp11;
		gboolean _tmp10;
		GPid _tmp9;
		gint _tmp8_length1;
		char** _tmp8;
		char** _tmp7;
		const char* _tmp6;
		const char* _tmp5;
		gboolean _tmp14;
		command = g_strdup ("valac");
		vtg_project_manager_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start compiling file: %s\n"), filename);
		vtg_project_manager_output_view_log_message (log, start_message);
		_tmp3 = NULL;
		_tmp2 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp3 = g_strdup_printf ("%s\n\n", _tmp2 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp3 = (g_free (_tmp3), NULL);
		_tmp2 = (g_free (_tmp2), NULL);
		_tmp4 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp4 = g_strdup_printf ("%s %s\n", command, filename));
		_tmp4 = (g_free (_tmp4), NULL);
		_tmp8 = NULL;
		_tmp7 = NULL;
		_tmp6 = NULL;
		_tmp5 = NULL;
		_tmp10 = g_spawn_async_with_pipes (working_dir, (_tmp8 = (_tmp7 = g_new0 (char*, 2 + 1), _tmp7[0] = (_tmp5 = command, (_tmp5 == NULL) ? NULL : g_strdup (_tmp5)), _tmp7[1] = (_tmp6 = filename, (_tmp6 == NULL) ? NULL : g_strdup (_tmp6)), _tmp7), _tmp8_length1 = 2, _tmp8), NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &_tmp9, NULL, &stdo, &stde, &inner_error);
		child_pid = (_tmp11 = (_tmp12 = &_tmp9, (_tmp12 == NULL) ? NULL : _g_pid_dup (_tmp12)), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp11);
		_tmp10;
		if (inner_error != NULL) {
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch9_g_spawn_error;
			}
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		_tmp8 = (_vala_array_free (_tmp8, _tmp8_length1, (GDestroyNotify) g_free), NULL);
		if (child_pid != NULL) {
			self->priv->_child_watch_id = g_child_watch_add (*child_pid, _vtg_project_manager_builder_on_child_watch_gchild_watch_func, self);
			vtg_project_manager_build_log_view_initialize (self->priv->_build_view, NULL);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp13;
				self->priv->is_bottom_pane_visible = (g_object_get (G_OBJECT ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin))), "visible", &_tmp13, NULL), _tmp13);
			}
			vtg_project_manager_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_project_manager_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			vtg_project_manager_output_view_log_message (log, "error compiling file\n");
		}
		return (_tmp14 = TRUE, command = (g_free (command), NULL), start_message = (g_free (start_message), NULL), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), _tmp14);
	}
	goto __finally9;
	__catch9_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			char* msg;
			gboolean _tmp15;
			msg = g_strdup_printf ("error spawning compiler process: %s", err->message);
			g_warning (msg);
			vtg_project_manager_output_view_log_message (log, msg);
			return (_tmp15 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), msg = (g_free (msg), NULL), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), _tmp15);
		}
	}
	__finally9:
	;
	working_dir = (g_free (working_dir), NULL);
	(child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL));
	(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
}


gboolean vtg_project_manager_builder_build (VtgProjectManagerBuilder* self, VtgProjectManagerProject* project) {
	GError * inner_error;
	const char* _tmp1;
	char* working_dir;
	GPid* child_pid;
	gint stdo;
	gint stde;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	_tmp1 = NULL;
	working_dir = (_tmp1 = project->filename, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	child_pid = 0;
	stdo = 0;
	stde = 0;
	{
		VtgProjectManagerOutputView* _tmp2;
		VtgProjectManagerOutputView* log;
		char* start_message;
		char* _tmp4;
		char* _tmp3;
		char* _tmp5;
		GPid* _tmp11;
		GPid* _tmp10;
		gboolean _tmp9;
		GPid _tmp8;
		gint _tmp7_length1;
		char** _tmp7;
		char** _tmp6;
		gboolean _tmp13;
		_tmp2 = NULL;
		log = (_tmp2 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		vtg_project_manager_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start building project: %s\n"), project->name);
		vtg_project_manager_output_view_log_message (log, start_message);
		_tmp4 = NULL;
		_tmp3 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp4 = g_strdup_printf ("%s\n\n", _tmp3 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp4 = (g_free (_tmp4), NULL);
		_tmp3 = (g_free (_tmp3), NULL);
		_tmp5 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp5 = g_strdup_printf ("%s\n", VTG_PROJECT_MANAGER_BUILDER_MAKE));
		_tmp5 = (g_free (_tmp5), NULL);
		_tmp7 = NULL;
		_tmp6 = NULL;
		_tmp9 = g_spawn_async_with_pipes (working_dir, (_tmp7 = (_tmp6 = g_new0 (char*, 1 + 1), _tmp6[0] = g_strdup (VTG_PROJECT_MANAGER_BUILDER_MAKE), _tmp6), _tmp7_length1 = 1, _tmp7), NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &_tmp8, NULL, &stdo, &stde, &inner_error);
		child_pid = (_tmp10 = (_tmp11 = &_tmp8, (_tmp11 == NULL) ? NULL : _g_pid_dup (_tmp11)), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp10);
		_tmp9;
		if (inner_error != NULL) {
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch10_g_spawn_error;
			}
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		_tmp7 = (_vala_array_free (_tmp7, _tmp7_length1, (GDestroyNotify) g_free), NULL);
		if (child_pid != NULL) {
			self->priv->_child_watch_id = g_child_watch_add (*child_pid, _vtg_project_manager_builder_on_child_watch_gchild_watch_func, self);
			vtg_project_manager_build_log_view_initialize (self->priv->_build_view, project);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp12;
				self->priv->is_bottom_pane_visible = (g_object_get (G_OBJECT ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin))), "visible", &_tmp12, NULL), _tmp12);
			}
			vtg_project_manager_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_project_manager_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			vtg_project_manager_output_view_log_message (log, "error spawning 'make' process\n");
		}
		return (_tmp13 = TRUE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp13);
	}
	goto __finally10;
	__catch10_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp14;
			g_warning ("vtgprojectmanagerbuilder.vala:136: Error spawning build process: %s", err->message);
			return (_tmp14 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp14);
		}
	}
	__finally10:
	;
	working_dir = (g_free (working_dir), NULL);
	(child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL));
}


gboolean vtg_project_manager_builder_clean (VtgProjectManagerBuilder* self, VtgProjectManagerProject* project, gboolean vala_stamp) {
	GError * inner_error;
	const char* _tmp1;
	char* working_dir;
	GPid* child_pid;
	gint stdo;
	gint stde;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project != NULL, FALSE);
	inner_error = NULL;
	if (self->priv->_child_watch_id != 0) {
		return FALSE;
	}
	_tmp1 = NULL;
	working_dir = (_tmp1 = project->filename, (_tmp1 == NULL) ? NULL : g_strdup (_tmp1));
	child_pid = 0;
	stdo = 0;
	stde = 0;
	{
		VtgProjectManagerOutputView* _tmp2;
		VtgProjectManagerOutputView* log;
		char* start_message;
		char* _tmp4;
		char* _tmp3;
		char* _tmp10;
		GPid* _tmp16;
		GPid* _tmp15;
		gboolean _tmp14;
		GPid _tmp13;
		gint _tmp12_length1;
		char** _tmp12;
		char** _tmp11;
		gboolean _tmp18;
		_tmp2 = NULL;
		log = (_tmp2 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
		vtg_project_manager_output_view_clean_output (log);
		start_message = g_strdup_printf (_ ("Start cleaning project: %s\n"), project->name);
		vtg_project_manager_output_view_log_message (log, start_message);
		_tmp4 = NULL;
		_tmp3 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp4 = g_strdup_printf ("%s\n\n", _tmp3 = g_strnfill ((gulong) (string_get_length (start_message) - 1), '-')));
		_tmp4 = (g_free (_tmp4), NULL);
		_tmp3 = (g_free (_tmp3), NULL);
		if (vala_stamp) {
			char* _tmp5;
			char* command;
			char* _tmp6;
			gboolean _tmp7;
			_tmp5 = NULL;
			vtg_project_manager_output_view_log_message (log, _tmp5 = g_strdup_printf (_ ("cleaning 'stamp' files for project: %s\n"), project->name));
			_tmp5 = (g_free (_tmp5), NULL);
			command = g_strdup_printf ("find %s -name *.stamp -delete", working_dir);
			_tmp6 = NULL;
			vtg_project_manager_output_view_log_message (log, _tmp6 = g_strdup_printf ("%s\n\n", command));
			_tmp6 = (g_free (_tmp6), NULL);
			_tmp7 = g_spawn_command_line_sync (command, NULL, NULL, NULL, &inner_error);
			if (inner_error != NULL) {
				if (inner_error->domain == G_SPAWN_ERROR) {
					goto __catch11_g_spawn_error;
				}
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
				g_clear_error (&inner_error);
			}
			if (!_tmp7) {
				char* _tmp8;
				gboolean _tmp9;
				_tmp8 = NULL;
				vtg_project_manager_output_view_log_message (log, _tmp8 = g_strdup_printf (_ ("error cleaning 'stamp' files for project: %s\n"), project->name));
				_tmp8 = (g_free (_tmp8), NULL);
				return (_tmp9 = FALSE, command = (g_free (command), NULL), (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp9);
			}
			command = (g_free (command), NULL);
		}
		_tmp10 = NULL;
		vtg_project_manager_output_view_log_message (log, _tmp10 = g_strdup_printf ("%s %s\n", VTG_PROJECT_MANAGER_BUILDER_MAKE, "clean"));
		_tmp10 = (g_free (_tmp10), NULL);
		_tmp12 = NULL;
		_tmp11 = NULL;
		_tmp14 = g_spawn_async_with_pipes (working_dir, (_tmp12 = (_tmp11 = g_new0 (char*, 2 + 1), _tmp11[0] = g_strdup (VTG_PROJECT_MANAGER_BUILDER_MAKE), _tmp11[1] = g_strdup ("clean"), _tmp11), _tmp12_length1 = 2, _tmp12), NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &_tmp13, NULL, &stdo, &stde, &inner_error);
		child_pid = (_tmp15 = (_tmp16 = &_tmp13, (_tmp16 == NULL) ? NULL : _g_pid_dup (_tmp16)), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp15);
		_tmp14;
		if (inner_error != NULL) {
			if (inner_error->domain == G_SPAWN_ERROR) {
				goto __catch11_g_spawn_error;
			}
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		_tmp12 = (_vala_array_free (_tmp12, _tmp12_length1, (GDestroyNotify) g_free), NULL);
		if (child_pid != NULL) {
			self->priv->_child_watch_id = g_child_watch_add (*child_pid, _vtg_project_manager_builder_on_child_watch_gchild_watch_func, self);
			vtg_project_manager_build_log_view_initialize (self->priv->_build_view, project);
			if (self->priv->last_exit_code == 0) {
				gboolean _tmp17;
				self->priv->is_bottom_pane_visible = (g_object_get (G_OBJECT ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin))), "visible", &_tmp17, NULL), _tmp17);
			}
			vtg_project_manager_output_view_start_watch (log, self->priv->_child_watch_id, stdo, stde, -1);
			vtg_project_manager_output_view_activate (log);
			g_signal_emit_by_name (self, "build-start");
		} else {
			vtg_project_manager_output_view_log_message (log, "error spawning 'make clean' process\n");
		}
		return (_tmp18 = TRUE, (log == NULL) ? NULL : (log = (g_object_unref (log), NULL)), start_message = (g_free (start_message), NULL), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp18);
	}
	goto __finally11;
	__catch11_g_spawn_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			gboolean _tmp19;
			g_warning ("vtgprojectmanagerbuilder.vala:181: Error spawning clean command: %s", err->message);
			return (_tmp19 = FALSE, (err == NULL) ? NULL : (err = (g_error_free (err), NULL)), working_dir = (g_free (working_dir), NULL), (child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL)), _tmp19);
		}
	}
	__finally11:
	;
	working_dir = (g_free (working_dir), NULL);
	(child_pid == NULL) ? NULL : (child_pid = (g_free (child_pid), NULL));
}


void vtg_project_manager_builder_next_error (VtgProjectManagerBuilder* self) {
	g_return_if_fail (self != NULL);
	vtg_project_manager_build_log_view_next_error (self->priv->_build_view);
}


void vtg_project_manager_builder_previous_error (VtgProjectManagerBuilder* self) {
	g_return_if_fail (self != NULL);
	vtg_project_manager_build_log_view_previous_error (self->priv->_build_view);
}


static void vtg_project_manager_builder_on_child_watch (VtgProjectManagerBuilder* self, GPid pid, gint status) {
	VtgProjectManagerOutputView* _tmp0;
	VtgProjectManagerOutputView* log;
	char* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp0 = NULL;
	log = (_tmp0 = vtg_plugin_get_output_view (self->priv->_plugin), (_tmp0 == NULL) ? NULL : g_object_ref (_tmp0));
	vtg_project_manager_output_view_stop_watch (log, self->priv->_child_watch_id);
	self->priv->last_exit_code = WEXITSTATUS (status);
	_tmp1 = NULL;
	vtg_project_manager_output_view_log_message (log, _tmp1 = g_strdup_printf (_ ("\ncompilation end with exit status %d\n"), self->priv->last_exit_code));
	_tmp1 = (g_free (_tmp1), NULL);
	vtg_project_manager_build_log_view_activate (self->priv->_build_view);
	self->priv->_child_watch_id = (guint) 0;
	g_signal_emit_by_name (self, "build-exit", self->priv->last_exit_code);
	g_spawn_close_pid (pid);
	if (self->priv->last_exit_code == 0) {
		if (!self->priv->is_bottom_pane_visible) {
			gtk_widget_hide ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin)));
		}
	} else {
		gdk_beep ();
	}
	(log == NULL) ? NULL : (log = (g_object_unref (log), NULL));
}


VtgPlugin* vtg_project_manager_builder_get_plugin (VtgProjectManagerBuilder* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_plugin;
}


static void vtg_project_manager_builder_set_plugin (VtgProjectManagerBuilder* self, VtgPlugin* value) {
	VtgPlugin* _tmp2;
	VtgPlugin* _tmp1;
	g_return_if_fail (self != NULL);
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_plugin = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1)), (self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL)), _tmp2);
	g_object_notify ((GObject *) self, "plugin");
}


VtgProjectManagerBuildLogView* vtg_project_manager_builder_get_error_pane (VtgProjectManagerBuilder* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return self->priv->_build_view;
}


gboolean vtg_project_manager_builder_get_is_building (VtgProjectManagerBuilder* self) {
	g_return_val_if_fail (self != NULL, FALSE);
	return self->priv->_child_watch_id != 0;
}


static GObject * vtg_project_manager_builder_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectManagerBuilderClass * klass;
	GObjectClass * parent_class;
	VtgProjectManagerBuilder * self;
	klass = VTG_PROJECT_MANAGER_BUILDER_CLASS (g_type_class_peek (VTG_PROJECT_MANAGER_TYPE_BUILDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_MANAGER_BUILDER (obj);
	{
		VtgProjectManagerBuildLogView* _tmp0;
		gboolean _tmp1;
		_tmp0 = NULL;
		self->priv->_build_view = (_tmp0 = vtg_project_manager_build_log_view_new (self->priv->_plugin), (self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL)), _tmp0);
		self->priv->is_bottom_pane_visible = (g_object_get (G_OBJECT ((GtkWidget*) gedit_window_get_bottom_panel (vtg_plugin_get_gedit_window (self->priv->_plugin))), "visible", &_tmp1, NULL), _tmp1);
	}
	return obj;
}


static void vtg_project_manager_builder_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerBuilder * self;
	self = VTG_PROJECT_MANAGER_BUILDER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_BUILDER_PLUGIN:
		g_value_set_object (value, vtg_project_manager_builder_get_plugin (self));
		break;
		case VTG_PROJECT_MANAGER_BUILDER_ERROR_PANE:
		g_value_set_object (value, vtg_project_manager_builder_get_error_pane (self));
		break;
		case VTG_PROJECT_MANAGER_BUILDER_IS_BUILDING:
		g_value_set_boolean (value, vtg_project_manager_builder_get_is_building (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_builder_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerBuilder * self;
	self = VTG_PROJECT_MANAGER_BUILDER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_BUILDER_PLUGIN:
		vtg_project_manager_builder_set_plugin (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_builder_class_init (VtgProjectManagerBuilderClass * klass) {
	vtg_project_manager_builder_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerBuilderPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_builder_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_builder_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_manager_builder_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_builder_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_BUILDER_PLUGIN, g_param_spec_object ("plugin", "plugin", "plugin", VTG_TYPE_PLUGIN, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_BUILDER_ERROR_PANE, g_param_spec_object ("error-pane", "error-pane", "error-pane", VTG_PROJECT_MANAGER_TYPE_BUILD_LOG_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_BUILDER_IS_BUILDING, g_param_spec_boolean ("is-building", "is-building", "is-building", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("build_start", VTG_PROJECT_MANAGER_TYPE_BUILDER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
	g_signal_new ("build_exit", VTG_PROJECT_MANAGER_TYPE_BUILDER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
}


static void vtg_project_manager_builder_instance_init (VtgProjectManagerBuilder * self) {
	self->priv = VTG_PROJECT_MANAGER_BUILDER_GET_PRIVATE (self);
	self->priv->_build_view = NULL;
	self->priv->_child_watch_id = (guint) 0;
	self->priv->last_exit_code = 0;
}


static void vtg_project_manager_builder_finalize (GObject* obj) {
	VtgProjectManagerBuilder * self;
	self = VTG_PROJECT_MANAGER_BUILDER (obj);
	(self->priv->_plugin == NULL) ? NULL : (self->priv->_plugin = (g_object_unref (self->priv->_plugin), NULL));
	(self->priv->_build_view == NULL) ? NULL : (self->priv->_build_view = (g_object_unref (self->priv->_build_view), NULL));
	G_OBJECT_CLASS (vtg_project_manager_builder_parent_class)->finalize (obj);
}


static GType vtg_project_manager_builder_type_id = 0;
GType vtg_project_manager_builder_get_type (void) {
	return vtg_project_manager_builder_type_id;
}


GType vtg_project_manager_builder_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerBuilderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_builder_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerBuilder), 0, (GInstanceInitFunc) vtg_project_manager_builder_instance_init, NULL };
	vtg_project_manager_builder_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerBuilder", &g_define_type_info, 0);
	return vtg_project_manager_builder_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL)
			destroy_func (((gpointer*) array)[i]);
		}
		else
		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) array)[i]);
		}
	}
	g_free (array);
}




