/*
 *  vtgutils.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include "vtgutils.h"
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gee/arraylist.h>
#include <gee/collection.h>
#include "vtgprojectmanagerprojectpackage.h"




static char* string_substring (const char* self, glong offset, glong len);
static glong string_get_length (const char* self);
enum  {
	VTG_UTILS_DUMMY_PROPERTY
};
static gboolean vtg_utils__initialized = FALSE;
static GscProposal** vtg_utils__proposals = NULL;
static gint vtg_utils__proposals_length1;
static GeeList* vtg_utils__available_packages = NULL;
static GtkBuilder* vtg_utils__builder = NULL;
static void vtg_utils_initialize (void);
static void _g_list_free_g_free (GList* self);
static void vtg_utils_initialize_packages_cache (void);
static gpointer vtg_utils_parent_class = NULL;
static void vtg_utils_finalize (GObject* obj);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);



static char* string_substring (const char* self, glong offset, glong len) {
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_warn_if_fail (offset >= 0);
	} else {
		g_warn_if_fail (offset <= string_length);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_warn_if_fail ((offset + len) <= string_length);
	start = g_utf8_offset_to_pointer (self, offset);
	return g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
}


static glong string_get_length (const char* self) {
	g_return_val_if_fail (self != NULL, 0L);
	return g_utf8_strlen (self, -1);
}


gboolean vtg_string_utils_is_null_or_empty (const char* data) {
	gboolean _tmp0;
	_tmp0 = FALSE;
	if (data == NULL) {
		_tmp0 = TRUE;
	} else {
		_tmp0 = _vala_strcmp0 (data, "") == 0;
	}
	return _tmp0;
}


char* vtg_string_utils_replace (const char* data, const char* search, const char* replace) {
	GError * inner_error;
	g_return_val_if_fail (data != NULL, NULL);
	g_return_val_if_fail (search != NULL, NULL);
	g_return_val_if_fail (replace != NULL, NULL);
	inner_error = NULL;
	{
		char* _tmp0;
		GRegex* _tmp1;
		GRegex* regex;
		char* _tmp2;
		char* _tmp3;
		_tmp0 = NULL;
		_tmp1 = NULL;
		regex = (_tmp1 = g_regex_new (_tmp0 = g_regex_escape_string (search, -1), 0, 0, &inner_error), _tmp0 = (g_free (_tmp0), NULL), _tmp1);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch28_g_regex_error;
			}
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		_tmp2 = g_regex_replace_literal (regex, data, (glong) (-1), 0, replace, 0, &inner_error);
		if (inner_error != NULL) {
			if (inner_error->domain == G_REGEX_ERROR) {
				goto __catch28_g_regex_error;
			}
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
			g_clear_error (&inner_error);
		}
		_tmp3 = NULL;
		return (_tmp3 = _tmp2, (regex == NULL) ? NULL : (regex = (g_regex_unref (regex), NULL)), _tmp3);
	}
	goto __finally28;
	__catch28_g_regex_error:
	{
		GError * e;
		e = inner_error;
		inner_error = NULL;
		{
			g_assert_not_reached ();
			(e == NULL) ? NULL : (e = (g_error_free (e), NULL));
		}
	}
	__finally28:
	;
}


GtkBuilder* vtg_utils_get_builder (void) {
	GError * inner_error;
	GtkBuilder* _tmp2;
	inner_error = NULL;
	if (vtg_utils__builder == NULL) {
		GtkBuilder* _tmp0;
		_tmp0 = NULL;
		vtg_utils__builder = (_tmp0 = gtk_builder_new (), (vtg_utils__builder == NULL) ? NULL : (vtg_utils__builder = (g_object_unref (vtg_utils__builder), NULL)), _tmp0);
		{
			char* _tmp1;
			_tmp1 = NULL;
			gtk_builder_add_from_file (vtg_utils__builder, _tmp1 = vtg_utils_get_ui_path ("vtg.ui"), &inner_error);
			if (inner_error != NULL) {
				goto __catch29_g_error;
			}
			_tmp1 = (g_free (_tmp1), NULL);
		}
		goto __finally29;
		__catch29_g_error:
		{
			GError * err;
			err = inner_error;
			inner_error = NULL;
			{
				g_warning ("vtgutils.vala:65: initialize_ui: %s", err->message);
				(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
			}
		}
		__finally29:
		;
	}
	_tmp2 = NULL;
	return (_tmp2 = vtg_utils__builder, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2));
}


GscProposal** vtg_utils_get_proposal_cache (int* result_length1) {
	GscProposal** _tmp0;
	if (!vtg_utils__initialized) {
		vtg_utils_initialize ();
	}
	_tmp0 = NULL;
	return (_tmp0 = vtg_utils__proposals, *result_length1 = vtg_utils__proposals_length1, _tmp0);
}


char* vtg_utils_get_image_path (const char* id) {
	char* result;
	g_return_val_if_fail (id != NULL, NULL);
	result = g_build_filename (PACKAGE_DATADIR, "images", id, NULL);
	return result;
}


char* vtg_utils_get_ui_path (const char* id) {
	char* result;
	g_return_val_if_fail (id != NULL, NULL);
	result = g_build_filename (PACKAGE_DATADIR, "ui", id, NULL);
	return result;
}


static void vtg_utils_initialize (void) {
	GError * inner_error;
	inner_error = NULL;
	{
		GscProposal** _tmp0;
		GdkPixbuf* _tmp1;
		GdkPixbuf* _icon_generic;
		_tmp0 = NULL;
		vtg_utils__proposals = (_tmp0 = g_new0 (GscProposal*, VTG_UTILS_prealloc_count + 1), vtg_utils__proposals = (_vala_array_free (vtg_utils__proposals, vtg_utils__proposals_length1, (GDestroyNotify) g_object_unref), NULL), vtg_utils__proposals_length1 = VTG_UTILS_prealloc_count, _tmp0);
		_tmp1 = NULL;
		_icon_generic = (_tmp1 = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), GTK_STOCK_FILE, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK, &inner_error), (_tmp1 == NULL) ? NULL : g_object_ref (_tmp1));
		if (inner_error != NULL) {
			goto __catch30_g_error;
		}
		{
			gint idx;
			idx = 0;
			for (; idx < VTG_UTILS_prealloc_count; idx++) {
				GscProposal* obj;
				GscProposal* _tmp3;
				GscProposal* _tmp2;
				obj = gsc_proposal_new ("", "", _icon_generic);
				_tmp3 = NULL;
				_tmp2 = NULL;
				vtg_utils__proposals[idx] = (_tmp3 = (_tmp2 = obj, (_tmp2 == NULL) ? NULL : g_object_ref (_tmp2)), (vtg_utils__proposals[idx] == NULL) ? NULL : (vtg_utils__proposals[idx] = (g_object_unref (vtg_utils__proposals[idx]), NULL)), _tmp3);
				(obj == NULL) ? NULL : (obj = (g_object_unref (obj), NULL));
			}
		}
		vtg_utils__initialized = TRUE;
		(_icon_generic == NULL) ? NULL : (_icon_generic = (g_object_unref (_icon_generic), NULL));
	}
	goto __finally30;
	__catch30_g_error:
	{
		GError * err;
		err = inner_error;
		inner_error = NULL;
		{
			g_warning (err->message);
			(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
		}
	}
	__finally30:
	;
}


GeeList* vtg_utils_get_available_packages (void) {
	GeeList* _tmp0;
	if (vtg_utils__available_packages == NULL) {
		vtg_utils_initialize_packages_cache ();
	}
	_tmp0 = NULL;
	return (_tmp0 = vtg_utils__available_packages, (_tmp0 == NULL) ? NULL : gee_collection_object_ref (_tmp0));
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


static void vtg_utils_initialize_packages_cache (void) {
	GError * inner_error;
	GList* vapidirs;
	GeeList* _tmp0;
	inner_error = NULL;
	vapidirs = NULL;
	vapidirs = g_list_append (vapidirs, g_strdup ("/usr/share/vala/vapi"));
	vapidirs = g_list_append (vapidirs, g_strdup ("/usr/local/share/vala/vapi"));
	_tmp0 = NULL;
	vtg_utils__available_packages = (_tmp0 = (GeeList*) gee_array_list_new (VTG_PROJECT_MANAGER_TYPE_PROJECT_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), (vtg_utils__available_packages == NULL) ? NULL : (vtg_utils__available_packages = (gee_collection_object_unref (vtg_utils__available_packages), NULL)), _tmp0);
	{
		GList* vapidir_collection;
		GList* vapidir_it;
		vapidir_collection = vapidirs;
		for (vapidir_it = vapidir_collection; vapidir_it != NULL; vapidir_it = vapidir_it->next) {
			const char* _tmp9;
			char* vapidir;
			_tmp9 = NULL;
			vapidir = (_tmp9 = (const char*) vapidir_it->data, (_tmp9 == NULL) ? NULL : g_strdup (_tmp9));
			{
				GDir* dir;
				const char* _tmp3;
				char* filename;
				dir = NULL;
				{
					GDir* _tmp1;
					GDir* _tmp2;
					_tmp1 = g_dir_open (vapidir, 0, &inner_error);
					if (inner_error != NULL) {
						if (inner_error->domain == G_FILE_ERROR) {
							goto __catch31_g_file_error;
						}
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, inner_error->message);
						g_clear_error (&inner_error);
					}
					_tmp2 = NULL;
					dir = (_tmp2 = _tmp1, (dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL)), _tmp2);
				}
				goto __finally31;
				__catch31_g_file_error:
				{
					GError * err;
					err = inner_error;
					inner_error = NULL;
					{
						/*do nothing*/
						(err == NULL) ? NULL : (err = (g_error_free (err), NULL));
						vapidir = (g_free (vapidir), NULL);
						(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
						continue;
					}
				}
				__finally31:
				;
				_tmp3 = NULL;
				filename = (_tmp3 = g_dir_read_name (dir), (_tmp3 == NULL) ? NULL : g_strdup (_tmp3));
				while (filename != NULL) {
					char* _tmp8;
					const char* _tmp7;
					if (g_str_has_suffix (filename, ".vapi")) {
						char* _tmp4;
						VtgProjectManagerProjectPackage* _tmp6;
						char* _tmp5;
						_tmp4 = NULL;
						filename = (_tmp4 = g_utf8_strdown (filename, -1), filename = (g_free (filename), NULL), _tmp4);
						_tmp6 = NULL;
						_tmp5 = NULL;
						gee_collection_add ((GeeCollection*) vtg_utils__available_packages, _tmp6 = vtg_project_manager_project_package_new (_tmp5 = string_substring (filename, (glong) 0, string_get_length (filename) - 5)));
						(_tmp6 == NULL) ? NULL : (_tmp6 = (g_object_unref (_tmp6), NULL));
						_tmp5 = (g_free (_tmp5), NULL);
					}
					_tmp8 = NULL;
					_tmp7 = NULL;
					filename = (_tmp8 = (_tmp7 = g_dir_read_name (dir), (_tmp7 == NULL) ? NULL : g_strdup (_tmp7)), filename = (g_free (filename), NULL), _tmp8);
				}
				vapidir = (g_free (vapidir), NULL);
				(dir == NULL) ? NULL : (dir = (g_dir_close (dir), NULL));
				filename = (g_free (filename), NULL);
			}
		}
	}
	(vapidirs == NULL) ? NULL : (vapidirs = (_g_list_free_g_free (vapidirs), NULL));
}


VtgUtils* vtg_utils_construct (GType object_type) {
	VtgUtils * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgUtils* vtg_utils_new (void) {
	return vtg_utils_construct (VTG_TYPE_UTILS);
}


static void vtg_utils_class_init (VtgUtilsClass * klass) {
	vtg_utils_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vtg_utils_finalize;
}


static void vtg_utils_instance_init (VtgUtils * self) {
}


static void vtg_utils_finalize (GObject* obj) {
	VtgUtils * self;
	self = VTG_UTILS (obj);
	G_OBJECT_CLASS (vtg_utils_parent_class)->finalize (obj);
}


static GType vtg_utils_type_id = 0;
GType vtg_utils_get_type (void) {
	return vtg_utils_type_id;
}


GType vtg_utils_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgUtilsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_utils_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgUtils), 0, (GInstanceInitFunc) vtg_utils_instance_init, NULL };
	vtg_utils_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgUtils", &g_define_type_info, 0);
	return vtg_utils_type_id;
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		if (array_length >= 0)
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL)
			destroy_func (((gpointer*) array)[i]);
		}
		else
		for (i = 0; ((gpointer*) array)[i] != NULL; i = i + 1) {
			destroy_func (((gpointer*) array)[i]);
		}
	}
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




